/** The VDOM representation of an `HTMLElement`. */
export interface IVirtualNode<A = {}> {
    /** Name of node, for example `div`, `span`, `table`, etc... */
    nodeName: string;
    /** Attributes of virtual HTML node, like `href` in `<a href="#"></a>` */
    attributes?: A;
    /** Child HTML nodes, for example `<span>Text</span>` is child node of
     * ```html
     * <div>
     *   <span>Text</span>
     * </div>
     * ```
     */
    children: ChildVirtualNode[];
    /** This is the special attribute that helps to identify nodes while DOM updates */
    key?: string;
}
/** This is a lazy analog of `IVirtualNode`. The difference between two is that the lazy
 *  virtual node calculates its properties immediately before rendering of the corresponding
 *  DOM node, while any instance of `IVirtualNode` is known in the very beginning of the rendering.
 */
export declare type LazyVirtualNode = () => IVirtualNode;
/** Plain component is a function that returns a custom `IVirtualNode`. */
export declare type PlainComponent<A = {}> = (attributes: A, children: ChildVirtualNode[]) => IVirtualNode<A>;
/** Lazy component function that returns a custom `LazyVirtualNode`. */
export declare type LazyComponent<A = {}> = (attributes: A, children: ChildVirtualNode[]) => LazyVirtualNode;
/** A component can be plain or lazy. */
export declare type Component<A = {}> = LazyComponent<A> | PlainComponent<A>;
/** Possible types of child nodes of VDOM tree. */
export declare type ChildVirtualNode<A = {}> = IVirtualNode<A> | string | number | null;
/** Type of JSX Factory's `children` argument. */
export declare type ChildLike = ChildVirtualNode | ChildVirtualNode[];
/** A JSX factory function that creates an `IVirtualNode` based on a given `name`, which
 * can be of type `string` or `PlainComponent`, an optional `attributes` and a `children`
 * argument  that describes some set of child virtual nodes.
 */
export declare function h<A>(name: string | PlainComponent<A>, attributes?: A, ...children: ChildLike[]): IVirtualNode<A>;
/** A JSX factory function that creates an `LazyVirtualNode` based on `comp` -- a given `LazyComponent`,
 *  an optional `attributes` and a `children` argument that describes some set of child virtual nodes.
 */
export declare function h<A>(comp: LazyComponent<A>, attributes?: A, ...children: ChildLike[]): LazyVirtualNode;
export declare type JSXFactory = typeof h;
